/* _____________________________________________________________________________
*
*	IO Module with Modbus Communication
* _____________________________________________________________________________
*
*	Target : ATmega328p
*	Clock  : Xtal 16.0 MHz
*
*	Author : Tom Viktora

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <string.h>
#include "ioctrl.h"
#include "iodef.h"
#include "std.h"
#include "MODBUS.h"


ISR(USART_RX_vect)
{	
	rx_byte = UDR0;
	rx_data[cnt_rx] = rx_byte;	

	if((cnt_rx == 1) && ((rx_byte == 0x0F) || (rx_byte == 0x10)))
		cahnge_data_lenght = 1;

	if((cnt_rx == 6) && (cahnge_data_lenght == 1))
	{
		cahnge_data_lenght = 0;
		modbus_data_lenght = 9 + rx_byte;		
	}

	cnt_rx++;

	if (cnt_rx == modbus_data_lenght)
	{
		check_rx_data = 1;
	}
}

// 115200bps/16MHz
void uart_init()
{
	UBRR0H = 0;
	//115200
	UBRR0L = 16;
	// double speed
	UCSR0A = 1<<U2X0;
	// 8bit
	UCSR0C = 1<<UCSZ01 | 1<<UCSZ00;
	// enable receiver, rx interrupt
	UCSR0B = 1<<RXEN0 | 1<<TXEN0 | 1<<RXCIE0; // | 1<<TXCIE0;	
}

void io_init()
{
	setin(IN1);
	setin(IN2);
	setin(IN3);
	setin(IN4);
	setout(OUT1);
	setout(OUT2);
	setout(OUT3);
	setout(OUT4);
	setout(TEN);
	setin(INT_I);
	setout(INT_O);
	setin(IN_MCU);
	setin(SW1);

	setin(RX);
	setout(TX);
	
	no_of_inputs = 4;
	no_of_outputs = 4;
	last_in_state = 0x00;
	is_change_out = 0;
	check_rx_data = 0;
	modbus_data_lenght = 8;

}

void clear_register()
{
	for(int i = 0; i < 4;i++)
	{
		digital_in[i] = 0x00;
		digital_out[i] = 0x00;
	}

	for(int i = 0; i < 8;i++)
	{
		analog_in[i] = 0x0000;
		analog_out[i] = 0x0000;
	}
}

void check_inputs()
{
	uchar temp = (PIND >> 3) & 0b00001111;

	if(last_in_state != temp)
	{
		last_in_state = temp;
		digital_in[0] = (digital_in[0] & 11110000) | ((~temp) & 0b00001111);
	}
}

void check_output_state()
{
	uchar temp = 0x00;

	if(is_change_out)
	{
		for(int i = 0; i < 4; i++)
		{
			temp = (digital_out[0] >> i) & 0b00000001;

			switch (i)
			{
				case 0:
				{
					write(OUT1,temp);
					break;
				}
				case 1:
				{
					write(OUT2,temp);
					break;
				}
				case 2:
				{
					write(OUT3,temp);
					break;
				}
				case 3:
				{
					write(OUT4,temp);
					break;
				}
				default:
					break;
			}
		}
		
		is_change_out = 0;
	}
}


int main(void)
{
	io_init();
	uart_init();
	clear_register();

	sei();
	//write(TEN, 1);	
		
	while (1)
	{	
		if (check_rx_data)
		{	
			check_rx_data = 0;
			modbus_data_lenght = 8;
			recieve_task();						
		}	

		check_inputs();
		check_output_state();
		
	}
}

